;********************************************************************************************
;* Title:  PACED_LOOP2.ASM                      (c) MOTOROLA Inc. 2003 All rights reserved.
;********************************************************************************************
;* Author: Bill Lucas - Motorola TSPG
;*
;* Description: Software paced loop demo
;*
;* This program is a software paced loop framework. It calls user tasks every 100
;* milliseconds. This implementation has five user tasks. The first maintains a
;* 0-60 second software clock. The next four user tasks sequence LEDs 1 through 4.
;*
;* Documentation: 9S08GB60 family Data Sheet and HCS08 Family Reference Manual (HCS08RM1/D)
;*
;* Include Files: None
;*
;* Assembler:  Metrowerks Code Warrior 3.0 (pre-release)
;*             or P&E Microcomputer Systems - CASMS08 (beta v4.02)
;*
;* Revision History:  not yet released
;* Rev #     Date       Who        Comments
;* -----  -----------  ------     --------------------------------------------
;*  1.3    07-May-03   J.Suchyta   Added LEDs 2 & 3
;*  1.2    25-Apr-03   B.Lucas     Correct minor typos in comments
;*  1.1    15-Apr-03   B.Lucas     Comments and modify for CW 3.0 project
;*  1.0    02-Apr-03   B.Lucas     Initial code release
;********************************************************************************************
             base  10t         ;change default to decimal

PACED_LOOP   equ           *
;
;* Set-up the timer to interrupt every 100ms on overflow
             ldhx   #TPM_COUNT    ;set mod = 14745
             sthx   TPM1MODH      ;overflow every 100 ms. 
;                                 = 18.874e6/128/14745
             mov    #(mTOIE|mCLKSA|mPS2|mPS1|mPS0),TPM1SC
;                                 ov int en,bus/128
;
             cli
;* Set-up I/O ports. We will use LED's 1 thru 4.
             mov    #(mPTFDD3|mPTFDD2|mPTFDD1|mPTFDD0),PTFD ;Port D bits 0-3 = 1
             mov    #(mPTFDD3|mPTFDD2|mPTFDD1|mPTFDD0),PTFDD ;port D bits 0-3 outputs
;
;* Initialize variables
             clr    Tick_100ms    ;initial value = 0
             clr    Tock_1sec     ;initial value = 0
;
;* MAIN - Beging of main program loop
;* Loop is executed once every 100ms. A pass through all major task 
;* routines takes less than lOOmS and then time is wasted until the
;* MSB of Tick_100ms is set (every 100ms). At each timer overflo
;* interrupt,the overflow flag is cleared. 
;
;* The variable Tick_100ms (lower 4 bits) keep track of lOOmS
;* periods. When the lower four bits of Tick_100ms increments from
;* 9 to 10 it is cleared to 0 and Tock_1sec is incremented.	
;
MAIN         equ    *
             brclr  MAIN_flag,Tick_100ms,MAIN ;until MAIN_flag set
             lda    Tick_100ms
             and    #FLAG_mask    ;clears Tick_100ms & saves bits 0:3
             inca
             sta    Tick_100ms
             cmp    #ONE_SECOND
             bne    go_user
             clr    Tick_100ms
go_user      equ    *             ;user tasks follow:
             jsr    TIME          ;user time keeping task
             jsr    blink_LED1    ;user blink LED1 task
             jsr    blink_LED2    ;user blink LED2 task
             jsr    blink_LED3    ;user blink LED3 task
             jsr    blink_LED4    ;user blink LED4 task
;
;* Other user tasks would go here
             bra    MAIN          ;back to the top for next ov
;
* TIME - Update TOCs
* If	TIC = 0, increment 0->59
* If	TIC not = 0, just skip whole routine
;
TIME         EQU    *             ;Update TOCs
             TST    Tick_100ms    ;Check for Tick_100ms=zero
             BNE    XTIME         ;If not; just exit
             INC    Tock_1sec     ;Tock_1sec=Tock_1sec+1
             LDA    #60	
             CMP    Tock_1sec     ;Did Tock_1sec -> 60 ?
             BNE    XTIME         ;If not; just exit
             CLR    Tock_1sec     ;Tock_1sec's rollover
XTIME        RTS                  ;Return from TIME
;
blink_LED1   EQU    *             ;update LED1
             LDA    Tock_1sec     ;Get second clock
             AND	#%00000011	  ;Mask 2 LSBs
             CMP	#$00		  ;First of 4?
             BNE    LED1OFF	      ;If not, turn off LED1
             BCLR   LED1,PTFD     ;Turn on LED1
             BRA    XBLINK1       ;Then exit
LED1OFF      BSET   LED1,PTFD     ;Turn off LED1
XBLINK1      RTS                  ;Return from BLINK LED1
;
blink_LED2   EQU    *             ;update LED2
             LDA    Tock_1sec     ;Get second clock
             AND	#%00000011	  ;Mask 2 LSBs
             CMP	#$01		  ;Second of 4?
             BNE    LED2OFF	      ;If not, turn off LED2
             BCLR   LED2,PTFD     ;Turn on LED2
             BRA    XBLINK2       ;Then exit
LED2OFF      BSET   LED2,PTFD     ;Turn off LED2
XBLINK2      RTS                  ;Return from BLINK LED2
;
blink_LED3   EQU    *             ;update LED3
             LDA    Tock_1sec     ;Get second clock
             AND	#%00000011	  ;Mask 2 LSBs
             CMP	#$02		  ;Third of 4?
             BNE    LED3OFF	      ;If not, turn off LED3
             BCLR   LED3,PTFD     ;Turn on LED3
             BRA    XBLINK3       ;Then exit
LED3OFF      BSET   LED3,PTFD     ;Turn off LED3
XBLINK3      RTS                  ;Return from BLINK LED3
;
blink_LED4   EQU	*             ;update LED4
             LDA    Tock_1sec     ;Get second clock
             AND	#%00000011	  ;Mask 2 LSBs
             CMP	#$03		  ;Last of 4?
             BNE    LED4OFF       ;If not, turn off LED4
             BCLR   LED4,PTFD     ;Turn on LED4
             BRA    XBLINK4       ;Then exit
LED4OFF      BSET   LED4,PTFD     ;Turn off LED4
XBLINK4      RTS                  ;Return from BLINK LED4
;
;* Timer overflow interrupt routine
TPM1_OV      equ    *
             lda    TPM1SC        ;clear the overflow flag
             bclr   TOF,TPM1SC    ;second half clearing overflow flag
             bset   7,Tick_100ms  ;set msb for main to run user code
             rti
	